#!/usr/bin/env python2.7
# coding=utf8

import os
import sys
from youdao.config import __version__
from youdao.entry import Youdao
from youdao.sqlsaver import SQLSaver

reload(sys)
sys.setdefaultencoding("utf8")

db_path = SQLSaver().db_path
youdao = Youdao()


map_target = {
    '--trans': '直接翻译',
    '--web': '网络翻译',
    '--basic': '基本释义',
    '--all': "翻译+基本释义",
    '--help': "显示帮助信息",
    '--clean': "清除数据库",
    "--version": "版本信息",
    '--debug': "调试模式",
    '--comp': "自动补全"
}

short = {
    '--trans': "-t",
    '--web': '-w',
    '--basic': '-b',
    '--all': '-a',
    '--help': '-h',
    '--clean': '-c',
    '--version': '-v',
    '--debug': '-d',
    '--comp': '-cp'
}


def help_menu():
    print("\n有道翻译终端程序\n")
    print("Usage:")
    print("  youdao <word | phrase | sentence> [args...]\t参数后置，查询翻译或解释")
    print("  youdao [args...] <word | phrase | sentence>\t参数前置，查询翻译或解释\n")
    for i in map_target:
        print("  {},{}\t{}".format(i, short[i], map_target[i]))

    print("\n输入\033[01;31myoudao + 想要查询的内容\033[00m即可\n")
    print("更多帮助信息 \nhttps://github.com/hellflame/youdao/blob/v{}/README.md\n".format(__version__))


def main():
    argv = sys.argv
    arg_len = len(argv)
    if arg_len == 1:
        help_menu()
    else:
        if arg_len == 2:
            arg = argv[-1]
            if arg in ('-v', '--version'):
                print('YoudaoDict Version {}'.format(__version__))
            elif arg in ('-c', '--clean'):
                if os.path.exists(db_path):
                    os.remove(db_path)
                    print('`{}`  removed'.format(db_path))
                else:
                    print('`{}` not exists, doing nothing'.format(db_path))
            elif arg in ('-h', '--help'):
                help_menu()
            elif arg in ('-cp', '--comp'):
                print(youdao.complete_code())
            elif arg == '--shard':
                print(youdao.shred_auto_complete(''))
            else:
                if not arg.startswith("-"):
                    youdao.set_phrase(arg)
                    youdao.executor()
                    result = youdao.basic()
                    web_result = youdao.web()
                    trans = youdao.trans()
                    if result:
                        print(result)
                    elif not result and youdao.valid and web_result:
                        print(web_result)
                    elif not result and youdao.valid and trans:
                        print(trans)
                    else:
                        print(" (╯▔皿▔ )╯ \033[01;31m{}\033[00m ㄟ(▔皿▔ ㄟ)".format(arg))
                else:
                    help_menu()
        elif arg_len == 3:
            arg = argv[1:]
            if arg[0].startswith('-'):
                if not arg[0] == '--shard' and not arg[1].startswith("-"):
                    youdao.set_phrase(arg[1])
                    youdao.executor()

                    if arg[0] in ('-d', '--debug'):
                        print(youdao.check_raw())
                    elif arg[0] in ('-w', '--web'):
                        print(youdao.web())
                    elif arg[0] in ('-t', '--trans'):
                        print(youdao.trans())
                    elif arg[0] in ('-b', '--basic'):
                        print(youdao.basic())
                    elif arg[0] in ('-a', '--all'):
                        print(youdao.basic())
                        print(youdao.web())
                        print(youdao.trans())
                    elif arg[0] in {'-c', '--clean'}:
                        SQLSaver().remove_query(arg[1])
                        print("removed `{}`".format(arg[1]))
                    else:
                        help_menu()
                elif arg[0] == '--shard':
                    print(youdao.shred_auto_complete(arg[1]))

            elif arg[1].startswith('-'):
                if not arg[0].startswith('-'):
                    youdao.set_phrase(arg[0])
                    youdao.executor()
                    if arg[1] in ('-d', '--debug'):
                        print(youdao.check_raw())
                    elif arg[1] in ('-w', '--web'):
                        print(youdao.web())
                    elif arg[1] in ('-t', '--trans'):
                        print(youdao.trans())
                    elif arg[1] in ('-b', '--basic'):
                        print(youdao.basic())
                    elif arg[1] in ('-a', '--all'):
                        print(youdao.basic())
                        print(youdao.web())
                        print(youdao.trans())
                    else:
                        help_menu()
                else:
                    help_menu()
            else:
                temp = " ".join(arg)
                youdao.set_phrase(temp.strip('-'))
                youdao.executor()
                result = youdao.basic()
                if result:
                    print(result)
                elif not result and not youdao.is_new and youdao.valid:
                    print(youdao.trans())
                else:
                    print(" (╯▔皿▔ )╯ \033[01;31m{}\033[00m ㄟ(▔皿▔ ㄟ)".format(youdao.phrase))

        elif arg_len == 4:
            arg = argv[1:]
            temp = " ".join(arg)
            youdao.set_phrase(temp.strip('-'))
            youdao.executor()
            result = youdao.basic()
            if result:
                print(result)
            elif youdao.result and not youdao.is_new and not result and youdao.valid:
                print(youdao.trans())
            else:
                print(" (╯▔皿▔ )╯ \033[01;31m{}\033[00m ㄟ(▔皿▔ ㄟ)".format(youdao.phrase))


if __name__ == '__main__':
    main()




# coding=utf8

import os
import json
import sqlite3
from contextlib import contextmanager
from urllib import quote, unquote

from youdao.config import DB_PATH


class SQLSaver(object):
    def __init__(self, db_path=''):
        if db_path:
            self.db_path = db_path
        else:
            self.db_path = os.path.expanduser(DB_PATH)
        self.TABLE = 'query'

    @contextmanager
    def connection(self):
        """connection instance to sqlite"""
        db = sqlite3.connect(self.db_path)
        db.text_factory = str
        cursor = db.cursor()
        cursor.execute("CREATE TABLE IF NOT EXISTS `{}` ("
                       "id INTEGER PRIMARY KEY AUTOINCREMENT, "
                       "query varchar(50) NOT NULL UNIQUE,"
                       "raw_json TEXT NOT NULL DEFAULT '')".format(
                        self.TABLE))
        yield cursor
        db.commit()
        db.close()

    def query(self, query):
        with self.connection() as cursor:
            cursor.execute("select raw_json from {} WHERE query = ? ".format(self.TABLE),
                           (query,))
            result = cursor.fetchone()
            if result:
                return json.loads(unquote(result[0]))
            return None

    def remove_query(self, query):
        """remove one saved query"""
        with self.connection() as cursor:
            cursor.execute("delete from {} where query = ?".format(self.TABLE),
                           (query,))

    def shred_query(self, shred):
        """query for auto complete"""
        with self.connection() as cursor:
            if shred:
                cursor.execute("""select query from {} WHERE query like ? limit 10""".format(self.TABLE),
                               (shred + "%",))
            else:
                cursor.execute("select query from {} order by id desc limit 10".format(self.TABLE))

            return cursor.fetchall()

    def upset(self, query, raw_dict):
        """update or insert one query result"""
        with self.connection() as cursor:
            cursor.execute("select id from {} WHERE query = ? ".format(self.TABLE),
                           (query,))
            result = cursor.fetchone()
            raw_json = quote(json.dumps(raw_dict))
            if result:
                cursor.execute("update {} set raw_json = ? WHERE id = ?".format(self.TABLE),
                               (raw_json, result[0]))
            else:
                cursor.execute("insert into {} (query ,raw_json) VALUES (?, ?)".format(self.TABLE),
                               (query, raw_json))








# coding=utf8
import sys
from gevent.monkey import patch_all
from gevent.pool import Pool
from gevent.lock import BoundedSemaphore
from gevent.timeout import Timeout

from youdao.spider import Spider
from youdao.customizer import Customize
from youdao.sqlsaver import SQLSaver

patch_all()
reload(sys)
sys.setdefaultencoding("utf8")


class Race(object):
    def __init__(self, phrase=''):
        self.phrase = phrase.lower()
        self.pool = Pool()
        self.sem = BoundedSemaphore(3)
        self.result = None
        self.sql_saver = SQLSaver()

    def race(self, runner):
        """
        let runners run for themselves
        :param runner: callable runner
        """
        with self.sem:
            try:
                runner()
            except Timeout:  # 继承自 BaseException，不能通过 Exception 截获异常
                pass

    def local_sql_fetch(self):
        """fetch local cache from sqlite"""
        with Timeout(1, False):
            result = self.sql_saver.query(self.phrase)
            if result:
                self.racer_weapon(result, gun='sql')

    def custom_server_fetch(self):
        """fetch result from customer server"""
        with Timeout(5, False):
            result = Customize(self.phrase).server_fetch()
            if result:
                self.racer_weapon(result, gun='custom')

    def official_server_fetch(self):
        """fetch result from official site"""
        timeout = 7
        with Timeout(timeout, False):
            _, result = Spider(timeout=timeout).deploy(self.phrase)
            if result:
                self.racer_weapon(result, gun='official')

    def racer_weapon(self, bullet, gun):
        """
        bullet to stop the race
        :param bullet: result for the game
        :param gun: winner name
        :return:
        """
        self.result = bullet
        if not gun == 'sql' and 'possibles' not in bullet:
            self.sql_saver.upset(self.phrase, bullet)
        self.pool.kill()

    def launch_race(self):
        """start race"""
        self.pool.map(self.race, [
            self.custom_server_fetch,
            self.official_server_fetch,
            self.local_sql_fetch
        ])



